<?php
/**
 * Scheduling functionality similiar to the unix crontab.
 *
 * This class is used to set a time period when something should be
 * executed. The code needs to be combined with other code that will
 * trigger the PhpCronTab::isTimeToExecute() atleast once a minute.
 * For example in unix crontab.
 *
 * The following example will do database check everyday
 * day 21:00, and optimize database once a week.
 *
 * /var/www/batches/index.php
 * @code
 * include_once('FareOfficeLib/FareOfficeLib.php');
 * $phpCronTab = new PhpCronTab;
 * $phpCronTab->addUnixFormat(EVERY_DAY);
 * if ($phpCronTab->isTimeToExecute())
 * {
 *   // Check table user fast;
 * }
 *
 * $phpCronTab->clearUnixFormat()
 * $phpCronTab->addUnixFormat(EVERY_WEEK);
 * if ($phpCronTab->isTimeToExecute())
 * {
 *   // optimize table user;
 * }
 * @endcode
 *
 * Then add this to /etc/crontab
 * @code
 * 0 21 * * *	root /usr/bin/php /var/www/batches/index.php
 * @endcode
 *
 * @author Daniel Lindh <daniel@fareoffice.com>
 * @ingroup general
 */
class PhpCronTab
{
	/**
	 * Parameters for PhpCronTab::addUnixFormat()
	 */
	const EVERY_MINUTE 		= '*/1 * * * *';
	const EVERY_5_MINUTE 	= '*/5 * * * *';
	const EVERY_15_MINUTE = '*/15 * * * *';
	const EVERY_30_MINUTE = '*/30 * * * *';
	const EVERY_HOUR 			= '* */1 * * *';
	const EVERY_DAY 			= '0 21 * * *';
	const EVERY_WEEK 			= '0 22 * * 1';

	/**
	 * Constructor
	 */
	public function __construct()
	{
		$this->scheduleList = array();
		$this->setCurrentTime(time());
	}

	/**
	 * Add a new time period when isTimeExecuted should be return true.
	 *
	 * Each add follows a particular format as a series of fields,
	 * separated by spaces and/or tabs. Each field can have a single
	 * value or a series of values.
	 *
	 * OPERATORS
	 * There are several ways of specifying multiple date/time values in a
	 * field:
	 *
	 * - The comma (',') operator specifies a list of values,
	 *   for example: "1,3,4,7,8"
	 * - The dash ('-') operator specifies a range of values,
	 *   for example: "1-6", which is equivalent to "1,2,3,4,5,6"
	 * - The asterisk ('*') operator specifies all possible values for a
	 *   field. For example, an asterisk in the hour time field would
	 *   be equivalent to 'every hour'.
	 * - There is also an operator which some extended versions of
	 *   cron support, the slash ('/') operator, which can be used to
	 *   skip a given number of values. For example, "* /3" in the hour
	 *   time field is equivalent to "0,3,6,9,12,15,18,21";
	 *   "*" specifies 'every hour' but the "/3" means that only the
	 *   first, fourth, seventh... and such values given by "*" are used.
	 *
	 * Fields
	 * <code>
	 * +---------------- minute (0 - 59)
	 * |  +------------- hour (0 - 23)
	 * |  |  +---------- day of month (1 - 31)
	 * |  |  |  +------- month (1 - 12)
	 * |  |  |  |  +---- day of week (0 - 7) (Sunday=0 or 7)
	 * |  |  |  |  |
	 * *  *  *  *  *
	 * </code>
	 *
	 * Each of the patterns from the five fields may be either * (an
	 * asterisk), meaning all legal values, or a list of elements
	 * separated by commas.
	 *
	 * For "day of the week" (field 5), 0 are considered Sunday.
	 *
	 * Counterintuitively, if both "day of month" (field 3) and
	 * "day of week" (field 5) are present on the same line, then the
	 * command is executed when either is true.
	 *
	 * @param $defined_
	 * 				See the function comment.
	 */
	public function addUnixFormat($defined_)
	{
		$arr = explode(' ', $defined_);

		// @todo
		//assertLog(count($arr) == 5, EL_LEVEL_3, ECAT_DIAGNOSTIC, 'Not the right number of arguments: ' . count($arr) . ' ' . $defined_);

		$this->addUnixFormatEx($arr[0], $arr[1], $arr[2], $arr[3], $arr[4]);
	}

	/**
	 * See addUnixFormat
	 */
	public function addUnixFormatEx($minute_, $hour_, $dayOfMonth_, $month_, $dayOfWeek_)
	{
		$this->scheduleList[] = array
		(
			'minute' 				=> ltrim($minute_, '0'),
			'hour' 					=> ltrim($hour_, '0'),
			'day_of_month' 	=> ltrim($dayOfMonth_, '0'),
			'month' 				=> ltrim($month_, '0'),
			'day_of_week'	 	=> ltrim($dayOfWeek_, '0')
		);
	}

	/**
	 * Clear all added schedules.
	 */
	public function clearUnixFormat()
	{
		$this->scheduleList = array();
	}

	/**
	 * Check if it's time to execute the program.
	 *
	 * @return bool
	 */
	public function isTimeToExecute()
	{
		if (!empty($this->scheduleList))
		{
			$currentTime = $this->_getCurrentTimeArr();

			foreach ($this->scheduleList as $schedule)
			{
				if
				(
					$this->_checkClock($currentTime['minute'], $schedule['minute']) &&
					$this->_checkClock($currentTime['hour'], $schedule['hour']) &&
					$this->_checkClock($currentTime['day_of_month'], $schedule['day_of_month']) &&
					$this->_checkClock($currentTime['month'], $schedule['month']) &&
					$this->_checkClock($currentTime['day_of_week'], $schedule['day_of_week'])
				)
				{
					return true;
				}
			}
			return false;
		}
		else
		{
			// If no sceduleList are configured, always execute
			return true;
		}
	}

	/**
	 * Set the current time measured in the number of seconds since
	 * the Unix Epoch (January 1 1970 00:00:00 GMT).
	 *
	 * @param $time_
	 */
	public function setCurrentTime($time_)
	{
		$this->_currentTime = $time_;
	}

	/**
	 * Returns the current time measured in the number of seconds since
	 * the Unix Epoch (January 1 1970 00:00:00 GMT).
	 */
	public function getCurrentTime()
	{
		return $this->_currentTime;
	}

	/**
	 * Return array with current time.
	 *
	 * Example return array.
	 * @code
	 * array
	 * (
	 *   'minute' => 23,
	 *   'hour'=> 14,
	 *   'day_of_month' = 2,
	 *   'month' => 2,
	 *   'day_of_week = 5'
	 * );
	 * @endcode
	 *
	 * Current time without leading zeroes and zero based.
	 * Hour is between 0-23 for example.
	 */
	private function _getCurrentTimeArr()
	{
		return array
		(
			'minute' 				=> ltrim(date('i', $this->getCurrentTime()), '0'),
			'hour' 					=> ltrim(date('G', $this->getCurrentTime()), '0'),
			'day_of_month' 	=> ltrim(date('d', $this->getCurrentTime()), '0'),
			'month' 				=> ltrim(date('n', $this->getCurrentTime()), '0'),
			'day_of_week'	 	=> ltrim(date('w', $this->getCurrentTime()), '0')
		);
	}

	/**
	 * Check the hour or minute field for crontab syntax.
	 *
	 * @param $currentClock_ - string -
	 * 				Either the current hour or current minute.
	 * 				Should not begin with 0.
	 *
	 * @param $testClock_ - string -
	 * 				The hour or minute when the execution should start.
	 *
	 * @return - bool - true if execution should be done.
	 */
	private function _checkClock($currentClock_, $testClock_)
	{
		$testClockArr = array();

		// * always execute this.
		if ($testClock_ == '*')
		{
			return true;
		}

		// If same value
		elseif ($currentClock_ == $testClock_)
		{
			return true;
		}

		// Ranged values, 1-10 means all values from 1 to 10.
		elseif (strpos($testClock_, '-') !== false)
		{
			$numberRange = explode('-', $testClock_);
			if
			(
				count($numberRange) == 2 &&
				$currentClock_ >= $numberRange[0] &&
				$currentClock_ <= $numberRange[1]
			)
			{
				return true;
			}
		}

		// Will rebuild to real values in an array.
		// */3 will be 0,3,6,9,12,15,18,21 etc.
		elseif (strpos($testClock_, '/') !== false)
		{
			$skipNumber = explode('/', $testClock_);
			if ($skipNumber[0] == '*')
			{
				for($i = 0; $i < 60; $i+=$skipNumber[1])
				{
					$testClockArr[$i] = true;
				}
			}
		}

		// 1,3,7,9 will be expanded to an array.
		elseif (strpos($testClock_, ',') !== false)
		{
			$testClockArr = explode(',', $testClock_);
			$testClockArr = array_flip($testClockArr);
		}

		//
		if (!empty($testClockArr) && array_key_exists($currentClock_,  $testClockArr))
		{
			return true;
		}

		return false;
	}

	/**
	 * Check if current day of month matches with the scheduled day of month.
	 *
	 * @param $dayOfMonth_ - string -
	 * 				Current day of month, 0-6, 0 is sunday.
	 *
	 * @param $scheduleDayOfMonth_ - string -
	 * 				The scheduled day of month.
	 *
	 * @return - bool - true if execution should be done.
	 */
	private function _checkDayOfMonth($dayOfMonth_, $scheduleDayOfMonth_)
	{
		if ('*' == $scheduleDayOfMonth_ || $dayOfMonth_ == $scheduleDayOfMonth_)
		{
			return true;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Check if current month matches with scheduled month.
	 *
	 * @param $currentMonth_ - string -
	 * 				Current month
	 *
	 * @param $scheduleDMonth_ - string -
	 * 				The scheduled month.
	 *
	 * @return - bool - true if execution should be done.
	 */
	private function _checkMonth($currentMonth_, $scheduleDMonth_)
	{
		if ('*' == $scheduleDMonth_ || $currentMonth_ == $scheduleDMonth_)
		{
			return true;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Check if current day of week matches with the scheduled day of week.
	 *
	 * @param $dayOfWeek_ - string -
	 * 				Current day of week, 0-6, 0 is sunday.
	 *
	 * @param $scheduledDayOfWeek_ - string -
	 * 				The scheduled day of week.
	 *
	 * @return - bool - true if execution should be done.
	 */
	private function _checkDayOfWeek($dayOfWeek_, $scheduledDayOfWeek_)
	{
		if ('*' == $scheduledDayOfWeek_ || $dayOfWeek_ == $scheduledDayOfWeek_)
		{
			return true;
		}
		else
		{
			return false;
		}
	}

	/**
	 * List of scheduled times.
	 */
	private $scheduleList;
}

?>
