<?php
/**
 * Functions to execute shell commands.
 *
 *	@author Daniel Lindh <daniel@fareoffice.com>
 *	@ingroup general
 *
 * @todo: Use escapeshellarg() or escapeshellcmd() to ensure that users cannot trick the system into executing arbitrary commands.
 */
class ShellCmd
{
	/**
	 * Execute any shell command.
	 *
	 * If any error occurs an exception will be sent.
	 *
	 * @code
	 * try
	 * {
	 *   ShellCmd::exec('whoami');
	 *   ShellCmd::exec("wc", "I may be synthetic, but I\'m not stupid.\n\n");
	 * }
	 * catch (Exception $e)
	 * {
	 *   return $e->getMessage();
	 * }
	 * @endcode
	 *
	 * @param	$command_ - string -
	 * 				The command that should be executed.
	 *
	 * @param $parameter_ - string -
	 * 				Any parameter that should be sent to the shell
	 * 				input, after the command has been executed.
	 * 				Ie. A password after a login command has been
	 * 				executed
	 *
	 * @return - string - The output generated by the command.
	 */
	public static function exec($command_, $parameter_ = NULL)
	{
	  $retval = '';
	  $error = '';

	  $descriptorspec = array
	  (
	    0 => array('pipe', 'r'),
	    1 => array('pipe', 'w'),
	    2 => array('pipe', 'w')
	  );
	  $pipes = array();

	  $resource = proc_open($command_, $descriptorspec, $pipes, null, $_ENV);
	  if (is_resource($resource))
	  {
	    $stdin = $pipes[0];
	    $stdout = $pipes[1];
	    $stderr = $pipes[2];

	    // Write to standard in
	    if (!empty($parameter_))
	    {
		    fwrite($stdin, $parameter_);
	    }
	    fclose($stdin);

	    // Get result from command
	    while (!feof($stdout))
	    {
	    	$retval .= fgets($stdout);
	    }
	    fclose($stdout);

	    // Get the errors back, if any
	    while (!feof($stderr))
	    {
	    	$error .= fgets($stderr);
	    }
	    fclose($stderr);

	    $exitCode =  proc_close($resource);
	    if (0 != $exitCode)
	    {
	    	$error .= ' Exit code: ' . $exitCode;
	    }
	  }

	  if (! empty($error))
	  {
	  	throw new Exception($error);
	  }
	  else
	  {
	 		return $retval;
	  }
	}

	/**
	 * Execute a shell command on a remoteserver, through SSH.
	 *
	 * @param	$remoteServer_ - string -
	 * 				Login and servername of remote server
	 * 				Ie. fareoffice@dpwww1.fareoffice.com
	 *
	 * @param $command_	- string -
	 * 				The command that should be executed on the remote server.
	 */
	public static function sshExec($remoteServer_, $command_)
	{
		return ShellCmd::exec('/usr/bin/ssh ' . $remoteServer_ . ' "' . $command_ . '"');
	}

	/**
	 * Get a file from a remote server through SSH.
	 *
	 * @param	$remoteServer_
	 * 				Login and servername of remote server
	 * 				Ie. fareoffice@dpwww1.fareoffice.com
	 *
	 * @param $from_
	 * 				Directory and file on remote server that should be copied.
	 * 				Ie. /home/fareoffice/file.txt
	 *
	 * @param $to_
	 * 				Directory and file on local server where the file should be
	 * 				copied to.
	 * 				Ie. /home/fareoffice/LocalFile.txt
	 */
	public static function sshGet($remoteServer_, $from_, $to_)
	{
		ShellCmd::exec('/usr/bin/scp ' . $remoteServer_ . ':' . $from_ . ' ' . $to_);
	}

	/**
	 * Put a file on a remote server through SSH.
	 *
	 * @param	$remoteServer_
	 * 				Login and servername of remote server
	 * 				Ie. fareoffice@dpwww1.fareoffice.com
	 *
	 * @param $from_
	 * 				Directory and file on local server that should be copied.
	 * 				Ie. /home/fareoffice/LocalFile.txt
	 *
	 * @param $to_
	 * 				Directory and file on remote server where
	 * 				the file should be placed.
	 * 				Ie. /home/fareoffice/file.txt
	 */
	public static function sshPut($remoteServer_, $from_, $to_)
	{
		ShellCmd::exec('/usr/bin/scp ' . $from_ . ' ' . $remoteServer_ .':' . $to_);
	}
}

?>
